//$Id: list.cc,v 1.19 2002/03/15 17:47:08 cactus Exp $ -*- c++ -*-

/* Guikachu Copyright (C) 2001-2002 RDI Gerg <cactus@cactus.rulez.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "list.h"
#include "list-canvas.h"

#include "form-editor.h"

using Guikachu::Widgets::List;
using namespace Guikachu::GUI::FormEditor;

List::List (Resources::Form   *owner,
	    const std::string &id):
    Widget (owner, id),
    ResizeableWidget (changed),
    AutoWidth (changed),
    editor (0),
    font (changed, 0),
    visible_items (changed, 4),
    usable (changed, true),
    items (changed, std::vector<std::string> ())
{
}

List::~List ()
{
    if (editor)
	delete editor;
}

Gtk::Widget *List::create_editor ()
{
    if (!editor)
	editor = new ListProperties (this);
    
    return editor->get_editor ();
}

WidgetCanvasItem* List::create_canvas_item (Gnome::CanvasGroup *parent_group)
{
    return new ListCanvasItem (this, parent_group);
}

bool List::load (StorageNode &node)
{
    /* Non-NULL default values */
    usable = false;
    
    for (StorageNode curr_node  = node.children ();
	 curr_node; curr_node++)
    {
	if (curr_node.name () == "pos")
	{
	    x = curr_node.get_prop<int> ("x");
	    y = curr_node.get_prop<int> ("y");

	    std::string width_str = curr_node.get_prop<std::string> ("width");
	    if (width_str == "auto")
	    {
		manual_width = false;
		width = 0;
	    } else {
		manual_width = true;
		width = atoi (width_str.c_str ());
	    }
	}
	else if (curr_node.name () == "usable")
	    usable = true;
	else if (curr_node.name () == "items")
	{
	    font = curr_node.get_prop<int> ("font");
	    visible_items = curr_node.get_prop<int> ("visible");
	    
	    /* Iterate items */
	    std::vector<std::string> items_vec;
	    
	    for (StorageNode item_node = curr_node.children ();
		 item_node; item_node++)
	    {
		if (item_node.name () == "item")
		    items_vec.push_back (item_node.get_content ());
		else
		    g_warning ("Error parsing `list' resource: "
			       "unexpected element `%s/%s'",
			       curr_node.name ().c_str (),
			       item_node.name ().c_str ());
	    }
	    
	    items = items_vec;
	}
	else
	    g_warning ("Error parsing `list' widget: "
		       "unexpected element `%s'",
		       curr_node.name ().c_str ());
    }

    changed ();
    
    return true;
}

void List::save (StorageNode &node) const
{
    StorageNode my_node;
    
    /* Position */
    my_node = node.add_node ("pos");
    my_node.set_prop ("x", x);
    my_node.set_prop ("y", y);

    /* Size */
    if (manual_width)
	my_node.set_prop ("width", width);
    else
	my_node.set_prop ("width", "auto");

    /* Usable */
    if (usable)
	node.add_node ("usable");

    /* List items */
    my_node = node.add_node ("items");
    my_node.set_prop ("font", font);
    my_node.set_prop ("visible", visible_items);
    
    const std::vector<std::string> &items_vec = items;
    for (std::vector<std::string>::const_iterator i = items_vec.begin ();
	 i != items_vec.end (); i++)
	my_node.add_node ("item", *i);
}

void List::save_rcp (std::ostream &ostr) const
{
    gchar *width_str;

    if (manual_width)
	width_str = g_strdup_printf ("%d", width ());
    else
	width_str = g_strdup ("AUTO");
    
    ostr << "  LIST";
    const std::vector<std::string> &items_vec = items;
    for (std::vector<std::string>::const_iterator i = items_vec.begin ();
	 i != items_vec.end (); i++)
	ostr << " \"" << *i << "\"";
    
    ostr << " ID " << id;
    ostr << " AT (" << x << " " << y << " " << width_str << " " << " AUTO)";
    ostr << " FONT " << font;
    if (usable)
	ostr << " USABLE";
    else
	ostr << " NONUSABLE";

    ostr << " VISIBLEITEMS " << visible_items;
    
    ostr << std::endl;

    g_free (width_str);
}

int List::get_display_height () const
{
    return get_font_height (font) * visible_items;
}

int List::get_auto_width () const
{
    int width = 18;
    if (items ().size ())
	width = get_string_width (font, items ()[0]) + 3;
    
    return width;
}
